const HtmlWebpackPlugin = require('html-webpack-plugin');
const invariant = require('invariant');
const path = require('path');
const webpack = require('webpack');

module.exports = options => (env, webpackOptions) => {
  const {
    domID = '.app-root',
    title = 'wub-preset-react',
    emitIndex = true,
    template = path.resolve(__dirname, 'template.ejs'),
    rootComponent,
  } = options;

  invariant(
    typeof domID === 'string' && domID !== '',
    'wub-preset-react expects domID option to be a valid ID'
  );

  invariant(
    typeof rootComponent === 'string' && path.isAbsolute(rootComponent),
    'wub-preset-react expects rootComponent option to be an absolute path'
  );

  invariant(
    typeof template === 'string' && path.isAbsolute(template),
    'wub-preset-react expects template option to be an absolute path to an EJS template.'
  );

  const plugins = [
    new webpack.DefinePlugin({
      __REACT_DOM_ID__: JSON.stringify(domID),
    }),
  ];

  const entry = [require.resolve('./entrypoint')];

  if (webpackOptions.hot || emitIndex) {
    // Emit an index.html when hot reloading
    plugins.push(
      new HtmlWebpackPlugin({
        inject: false,
        title,
        template,
        domID,
      })
    );

    // patch is a noop in production but might as well filter it out
    entry.unshift(require.resolve('react-hot-loader/patch'));
  }

  const projectDir = path.dirname(module.parent.filename);
  path.join(projectDir, rootComponent);

  module.exports = function(babelrc) {
    babelrc.presets.push(require.resolve());
    babelrc.env.hot.plugins.unshift();
    return babelrc;
  };

  return {
    entry,
    plugins,
    resolve: {
      alias: {
        __REACT_ROOT_COMPONENT__: rootComponent,
        react: path.dirname(require.resolve('react/package')),
        'react-dom': path.dirname(require.resolve('react-dom/package')),
      },
    },
    module: {
      rules: [
        {
          test: /\.jsx?$/,
          exclude: modulePath =>
            // it's in a node_modules folder
            modulePath.indexOf('node_modules') > -1 &&
            // and it's not a file required by a preset
            !/wub-preset-\w+\/[^/]+$/.test(modulePath),
          use: [
            {
              loader: require.resolve('babel-loader'),
              options: {
                presets: [
                  [
                    'env',
                    {
                      targets: {
                        browsers: [
                          '>1%',
                          'last 4 versions',
                          'not ie < 9',
                          'firefox esr',
                        ],
                      },
                      modules: false,
                    },
                  ],
                  require.resolve('babel-preset-react'),
                ],
                plugins: [
                  webpackOptions.hot &&
                    require.resolve('react-hot-loader/babel'),
                  'transform-object-rest-spread',
                  'transform-class-properties',
                  'syntax-dynamic-import',
                ].filter(Boolean),
              },
            },
          ],
        },
      ],
    },
  };
};
